@@ -32,9 +32,10 @@ import apt_pkg
 
 import pygtk
 pygtk.require("2.0")
-import gtk, gtk.glade
-import pango
+import glib
 import gobject
+import gtk
+import pango
 import vte
 import urllib
 import fcntl
@@ -44,7 +45,7 @@ import thread
 import re
 
 from DebPackage import DebPackage, Cache
-from SimpleGladeApp import SimpleGladeApp
+from SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from apt.progress import InstallProgress
 from GDebiCommon import GDebiCommon, utf8
 from gettext import gettext as _
@@ -53,17 +54,14 @@ from gettext import gettext as _
 # is happening 
 GDEBI_TERMINAL_TIMEOUT=4*60.0
 
-class GDebi(SimpleGladeApp, GDebiCommon):
+class GDebi(SimpleGtkbuilderApp, GDebiCommon):
 
     def __init__(self, datadir, options, file=""):
         GDebiCommon.__init__(self,datadir,options,file)
         localesApp="gdebi"
         localesDir="/usr/share/locale"
-        gtk.glade.bindtextdomain(localesApp, localesDir)
-        gtk.glade.textdomain(localesApp)
 
-        SimpleGladeApp.__init__(self, domain="gdebi",
-                                path=datadir+"/gdebi.glade")
+        SimpleGtkbuilderApp.__init__(self, path=datadir+"/gdebi.ui")
         # use a nicer default icon
         icons = gtk.icon_theme_get_default()
         try:
@@ -74,12 +72,6 @@ class GDebi(SimpleGladeApp, GDebiCommon):
           print "Error loading logo"
           pass
   
-        # set image of button "install"  manually, since it is overriden 
-        #by set_label otherwise
-        img = gtk.Image()
-        img.set_from_stock(gtk.STOCK_APPLY,gtk.ICON_SIZE_BUTTON)
-        self.button_install.set_image(img)
-
         # setup status
         self.context=self.statusbar_main.get_context_id("context_main_window")
         self.statusbar_main.push(self.context,_("Loading..."))
@@ -97,6 +89,7 @@ class GDebi(SimpleGladeApp, GDebiCommon):
 
         # show what we have
         self.window_main.show()
+        #self.vte_terminal.set_font_from_string("monospace 10")
         
         self.cprogress = self.CacheProgressAdapter(self.progressbar_cache)
         if not self.openCache():
@@ -112,7 +105,14 @@ class GDebi(SimpleGladeApp, GDebiCommon):
         column.add_attribute(render, "markup", 0)
         self.treeview_details.append_column(column)
         self.treeview_details.set_model(self.details_list)
-        
+
+        # setup the files treeview
+        column = gtk.TreeViewColumn("")
+        render = gtk.CellRendererText()
+        column.pack_start(render, True)
+        column.add_attribute(render, "text", 0)
+        self.treeview_files.append_column(column)
+
         if file != "" and os.path.exists(file):
             self.window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
             while gtk.events_pending(): gtk.main_iteration()        
@@ -134,6 +134,9 @@ class GDebi(SimpleGladeApp, GDebiCommon):
             path = path[5:] # 5 is len('file:')
         return path
     
+    def on_menuitem_quit_activate(self, widget):
+        gtk.main_quit()
+
     def on_window_main_drag_data_received(self, widget, context, x, y,
                                           selection, target_type, timestamp):
         """ call when we got a drop event """
@@ -156,20 +159,15 @@ class GDebi(SimpleGladeApp, GDebiCommon):
             
         self.statusbar_main.push(self.context, "")
 
-        # grey in since we are ready for user input now
-        self.window_main.set_sensitive(True)
-
         # set window title
         self.window_main.set_title(_("Package Installer - %s") % 
                                    self._deb.pkgName)
 
-        # set name
+        # set name and ungrey some widgets
         self.label_name.set_markup(self._deb.pkgName)
-        
         self.notebook_details.set_sensitive(True)
         self.hbox_main.set_sensitive(True)
 
-
         # set description
         buf = self.textview_description.get_buffer()
         try:
@@ -216,21 +214,37 @@ class GDebi(SimpleGladeApp, GDebiCommon):
         self.label_section.set_text(utf8(self._deb["Section"]))
         self.label_size.set_text(self._deb["Installed-Size"] + " KB")
 
-        # set filelist
-        buf = self.textview_filelist.get_buffer()
-        buf.set_text(utf8("\n".join(self._deb.filelist)))
+        # set file list
+        store = gtk.TreeStore(str)
+        try:
+            header = store.append(None, [_("Package control data")])
+            for name in self._deb.control_filelist:
+                store.append(header, [name])
+            header = store.append(None, [_("Upstream data")])
+            for name in self._deb.filelist:
+                store.append(header, [name])
+        except Exception, e:
+            print "Exception while reading the filelist: '%s'" % e
+            store.clear()
+            store.append(None, [_("Error reading filelist")])
+        self.treeview_files.set_model(store)
+        self.treeview_files.expand_all()
+        # and the file content textview
+        font_desc = pango.FontDescription('monospace')
+        self.textview_file_content.modify_font(font_desc)
 
         # check the deps
         if not self._deb.checkDeb():
             self.label_status.set_markup("<span foreground=\"red\" weight=\"bold\">"+
                                          _("Error: ") +
-                                         self._deb._failureString +
+                                         glib.markup_escape_text(self._deb._failureString) +
                                          "</span>")
 	    self.button_install.set_label(_("_Install Package"))
 
             self.button_install.set_sensitive(False)
             self.button_details.hide()
             return
+        
 
         # set version_info_{msg,title} strings
         self.compareDebWithCache()
@@ -262,13 +276,43 @@ class GDebi(SimpleGladeApp, GDebiCommon):
             self.button_details.show()
             
         self.label_status.set_markup(self.deps)
-        img = gtk.Image()
-        img.set_from_stock(gtk.STOCK_APPLY,gtk.ICON_SIZE_BUTTON)
-        self.button_install.set_image(img)
+        #img = gtk.Image()
+        #img.set_from_stock(gtk.STOCK_APPLY,gtk.ICON_SIZE_BUTTON)
+        #self.button_install.set_image(img)
         self.button_install.set_label(_("_Install Package"))
         self.button_install.set_sensitive(True)
         self.button_install.grab_default()
 
+    def on_treeview_files_cursor_changed(self, treeview):
+        " the selection in the files list chanaged "
+        model = treeview.get_model()
+        (path, col) = treeview.get_cursor()
+        name = model[path][0]
+        # if we are at the top-level, do nothing
+        if len(path) < 2:
+            return
+        # parent path == 0 means we look at the control information
+        # parent path == 1 means we look at the data
+        parent_path = path[0]
+        if name.endswith("/"):
+            data = _("Selection is a directory")
+        elif parent_path == 0:
+            try:
+                data = self._deb.control_content(name)
+            except Exception, e:
+                data = _("Error reading file content '%s'") % e
+        elif parent_path == 1:
+            try:
+                data = self._deb.data_content(name)
+            except Exception, e:
+                data = _("Error reading file content '%s'") % e
+        else:
+            assert False, "NOT REACHED"
+        if not data:
+            data = _("File content can not be extracted")
+        buf = self.textview_file_content.get_buffer()
+        buf.set_text(data)
+
     def on_button_details_clicked(self, widget):
         #print "on_button_details_clicked"
         # sanity check
@@ -286,6 +330,7 @@ class GDebi(SimpleGladeApp, GDebiCommon):
     def on_open_activate(self, widget):
         #print "open"
         # build dialog
+        self.window_main.set_sensitive(False)
         fs = gtk.FileChooserDialog(parent=self.window_main,
                                    buttons=(gtk.STOCK_CANCEL, 
                                             gtk.RESPONSE_CANCEL, 
@@ -305,6 +350,7 @@ class GDebi(SimpleGladeApp, GDebiCommon):
             #print fs.get_filename()
             self.open(fs.get_filename())
         fs.destroy()
+        self.window_main.set_sensitive(True)
 
     def on_refresh_activate(self, widget):
         #print "refresh"
@@ -339,10 +385,10 @@ class GDebi(SimpleGladeApp, GDebiCommon):
         self.statusbar_main.push(self.context,_("Installing package file..."))
         if widget != None and len(self.unauthenticated) > 0:
             primary = _("Install unauthenticated software?")
-            secondary = _("Malicous software can damage your data "
+            secondary = _("Malicious software can damage your data "
                           "and take control of your system.\n\n"
                           "The packages below are not authenticated and "
-                          "could therefor be of malicous nature.")
+                          "could therefor be of malicious nature.")
             msg = "<big><b>%s</b></big>\n\n%s" % (primary, secondary)
             dialog = gtk.MessageDialog(parent=self.dialog_deb_install,
                                        flags=gtk.DIALOG_MODAL,
@@ -389,7 +435,7 @@ Install software from trustworthy software distributors only.
         self.window_main.set_sensitive(False)
         self.button_deb_install_close.set_sensitive(False)
         # clear terminal
-        #self._term.feed(str(0x1b)+"[2J")
+        #self.vte_terminal.feed(str(0x1b)+"[2J")
         self.dialog_deb_install.set_transient_for(self.window_main)
         self.dialog_deb_install.show_all()
 
@@ -404,7 +450,7 @@ Install software from trustworthy software distributors only.
                                                 self.label_action,
                                                 self.dialog_deb_install)
             iprogress = self.InstallProgressAdapter(self.progressbar_install,
-                                                    self._term,
+                                                    self.vte_terminal,
                                                     self.label_action,
                                                     self.expander_install)
             errMsg = None
@@ -439,7 +485,7 @@ Install software from trustworthy software distributors only.
         dprogress = self.DpkgInstallProgress(self._deb.file,
                                              self.label_install_status,
                                              self.progressbar_install,
-                                             self._term,
+                                             self.vte_terminal,
                                              self.expander_install)
         dprogress.commit()
         self.install_completed=True
@@ -487,13 +533,6 @@ Install software from trustworthy software distributors only.
         else: 
             return True
 
-    def create_vte(self, arg1,arg2,arg3,arg4):
-        #print "create_vte (for the custom glade widget)"
-        self._term = vte.Terminal()
-        self._term.set_font_from_string("monospace 10")
-        return self._term
-
-
     def show_alert(self, type, header, body=None, details=None, parent=None):
         if parent is not None:
              self.dialog_hig.set_transient_for(parent)
@@ -622,7 +661,7 @@ Install software from trustworthy software distributors only.
                         "APT_LISTCHANGES_FRONTEND=gtk"]
         def child_exited(self,term, pid, status):
             #print "child_exited: %s %s %s %s" % (self,term,pid,status)
-            self.apt_status = posix.WEXITSTATUS(status)
+            self.apt_status = status
             self.finished = True
         def error(self, pkg, errormsg):
             # FIXME: display a msg
@@ -647,8 +686,16 @@ Install software from trustworthy software distributors only.
             if (not self.term_expander.get_expanded() and 
                 (self.time_last_update + GDEBI_TERMINAL_TIMEOUT) < time.time()):
               self.term_expander.set_expanded(True)
+            # sleep just long enough to not create a busy loop
+            time.sleep(0.01)
         def fork(self):
-            return self.term.forkpty(envv=self.env)
+            pid = self.term.forkpty(self.env)
+            if pid == 0:
+                # *grumpf* workaround bug in vte here (gnome bug #588871)
+                for env in self.env:
+                    (key, value) = env.split("=")
+                    os.environ[key] = value
+            return pid
         def waitChild(self):
             while not self.finished:
                 self.updateInterface()
@@ -707,7 +754,7 @@ Install software from trustworthy software distributors only.
 if __name__ == "__main__":
     app = GDebi("data/",None)
 
-    pkgs = ["3ddesktop"]
+    pkgs = ["cw"]
     for pkg in pkgs:
         print "installing %s" % pkg
         app._cache[pkg].markInstall()
@@ -721,8 +768,14 @@ if __name__ == "__main__":
     app.dialog_deb_install.show_all()
  
     # install the dependecnies
-    fprogress = app.FetchProgressAdapter(app.progressbar_install)
-    iprogress = app.InstallProgressAdapter(app.progressbar_install, app._term)
+    fprogress = app.FetchProgressAdapter(app.progressbar_install,
+                                         app.label_action,
+                                         app.dialog_deb_install)
+    iprogress = app.InstallProgressAdapter(app.progressbar_install, 
+                                           app.vte_terminal,
+                                           app.label_action,
+                                           app.expander_install)
     res = app._cache.commit(fprogress,iprogress)
     print "commit retured: %s" % res
     
+    gtk.main()
